package ca.ubc.cs.elide;

import java.util.*;
import java.lang.reflect.*;
import ca.ubc.cs.elide.*;
import ca.ubc.cs.elide.nodes.*;
import ca.ubc.cs.elide.transformations.*;

public class Transform
{
	private static Map transformInstances = new HashMap();
	private static List transformClasses = new ArrayList();
	
	private static TransformFactory factory;
		
	public static void setFactory(TransformFactory aFactory)
	{
		factory = aFactory;
	}
	
	public static void registerTransform(String transformName)
	{
		if(!transformClasses.contains(transformName))
			transformClasses.add(transformName);
	}
	
	private static Class findClass(String transformName)
	{
		Class transformClass = factory.findTransformClass(transformName);
		if(transformClass == null)
		{
			System.out.println("Could not find class for transform " + transformName);
			System.exit(1);	
		}
		return transformClass;
	}
	
	public static List resolveDependencies()
	{
		//flush singleton cache
		transformInstances = new HashMap();
		
		Set deps = new HashSet();
		
		for(int i = 0; i < transformClasses.size(); i++)
			deps.addAll(getDependencies(findClass((String) transformClasses.get(i))));

		return new Executor(deps).resolveAll();
	}

	public static Transform getInstance(String transformName)
	{
		return getInstance(findClass(transformName));
	}
	
	public static Set getDependencies(Class atomClass)
	{
		return (Set) invokePolymorphicStaticMethod(atomClass, "getDependencySet");
	}
	
	public static Set getDependencies(String className)
	{
		return getDependencies(findClass(className));
	}
	
	protected static Transform getInstance(Class atomClass)
	{
		return (Transform) invokePolymorphicStaticMethod(atomClass, "getInstanceObject");
	}
		
	protected static Object invokePolymorphicStaticMethod(Class aClass, String methodName)
	{
		try
		{
			Method method = aClass.getMethod(methodName, new Class[]{Class.class});
			return method.invoke(aClass, new Object[]{aClass});
		}
		catch(Exception e)
		{
			e.printStackTrace();
			System.out.println(e);
			System.exit(1);
		}
		
		return null;
	}
	
	public static Transform getInstanceObject(Class transformClass) 
	{
		Transform instance;
		if( (instance = (Transform)transformInstances.get(transformClass)) == null )
		{
			try
			{
				instance = (Transform)transformClass.newInstance();
				transformInstances.put(transformClass, instance);
			}
			catch(Exception e)
			{
				e.printStackTrace();
			}
		}
		
		return instance;
	}
	
	public static Set getDependencySet(Class transformClass)
	{
		System.out.println("no dependencies registered for " + transformClass);
		return new HashSet();
	}
}