package ca.ubc.cs.elide;

import ca.ubc.cs.elide.nodes.*;

import java.util.*;
import java.io.*;

public class Utils
{
	public static String methodCase(String name)
	{
		return name.substring(0, 1).toUpperCase() + name.substring(1);
	}
	
	public static String fieldCase(String name)
	{
		return name.substring(0,1).toLowerCase() + name.substring(1);
	}

	public static String firstStatement(String inString)
	{
		return inString.substring(0, inString.indexOf(";")+1);
	}
	
	public static boolean isConstructorWithCallToSuper(MethodNode method)
	{
		CodeNode cNode = method.getCodeElements()[0];
		return (method instanceof ConstructorNode) &&
		       (cNode instanceof TextNode) && 
			   ((TextNode)cNode).getText().trim().startsWith("super");
	}
	
	static int numberOfUniqueReps = 0;
	
	public static String uniqueRepresentation(String input)
	{
		return input + "_" + numberOfUniqueReps++; 
	}
	
	public static String join(List list, String spacer)
	{
		StringBuffer s = new StringBuffer();
		Iterator i = list.iterator();
		while(i.hasNext())
		{
			s.append(i.next());
			if(i.hasNext())
				s.append(spacer);
		}
		return s.toString();
	}
	
	public static String join(Object[] list, String spacer)
	{
		return join(Arrays.asList(list), spacer);
	}
	
	public static boolean isPrimitive(String type)
	{
		return type.equals("boolean") ||
			   type.equals("byte") ||
			   type.equals("char") ||
			   type.equals("short") ||
			   type.equals("int") ||
			   type.equals("long") ||
			   type.equals("float") ||
			   type.equals("double");
	}
	
	public static void each(List list, Lambda block)
	{
		Iterator i = list.iterator();
		while(i.hasNext()) block.apply(i.next());
	}
	
	public static List map(List list, Lambda block)
	{
		List result = new ArrayList();
		Iterator i = list.iterator();
		while(i.hasNext())
			result.add(block.apply(i.next()));
		return result;
	}
	
	public static void each(Object[] list, Lambda block)
	{
		each(Arrays.asList(list), block);
	}
	
	public static Object[] map(Object[] list, Lambda block)
	{
		return map(Arrays.asList(list), block).toArray();
	}
}