package ca.ubc.cs.elide.nodes;

import java.util.*;
import ca.ubc.cs.elide.utils.*;

/**
 * <code>CodeBearingNode</code> is an abstract class representing nodes which 
 * can contain Java code.
 */
public abstract class CodeBearingNode extends ModifierBearingNode
{
	public CodeBearingNode() {}
	public CodeBearingNode(String name) { super(name); }
	
	/**
	 * Returns the code elements associated with this <code>CodeBearingNode</code>
	 * as an array of {@link CodeNode}s
	 */
	public CodeNode[] getCodeElements()
	{
		return (CodeNode[]) getChildren(CodeNode.class);
	}
	
	/**
	 * Wraps the code elements of this <code>CodeBearingNode</code> object with
	 * code supplied by <code>node</code>.  <code>node</code> is used as a template,
	 * with the current code elements being substituted for the first occurrence of
	 * <code>separator</code>
	 */
    public void wrap(TextNode node, String separator)
	{
		String text = node.getText();
		int index = text.indexOf(separator);
		
		if(index == -1)
		{
			System.out.println("Warning: could not find '" + separator + "' in text to be wrapped");
			return;
		}
		
		String pre = text.substring(0, index);
		String post = text.substring(index + separator.length());
		
		add(0, new TextNode(pre));
		add(new TextNode(post));
	}

	public BlockNode findBlock(String name)
	{
		return (BlockNode) findChild(BlockNode.class, name);
	}
}