package ca.ubc.cs.elide.nodes;

import ca.ubc.cs.elide.*;
import ca.ubc.cs.elide.visitors.*;
import ca.ubc.cs.elide.utils.*;

/**
 * <code>FieldNode</code> objects represent fields of classes.
 * <code>FieldNode</code>s can be created by calling {@link NodeFactory#createField}
 * or by extending a class with new fields (see {@link ClassNode#extend})
 */
public class FieldNode extends ModifierBearingNode
{	
	public FieldNode() {}
		
	public FieldNode(String name, String type)
	{
		super(name);
		setType(type);
	}
	
	private String type;
	private String initializer;
		
	/**
	 * Returns a <code>String</code> representing the type of the field represented
	 * by this <code>FieldNode</code> object.
	 */
    public String getType() { return type; }
	
	/**
	 * Sets the type of the field represented by this <code>FieldNode</code> object.
	 * This method does not check whether the string provided names a valid type
	 * in the field's context.
	 */
	public void setType(String type) { this.type = type; }
	
	/**
	 * Returns a <code>String</code> representing the initialization statement
	 * for the field represented by this <code>FieldNode</code> object.
	 * An example would be <code>= new HashMap()</code>
	 *
	 * Returns an empty string if no intializer was provided.
	 */
    public String getInitializer() { return initializer; }
	
	/**
	 * Sets the initializer of the field represented by this <code>FieldNode</code> object
	 * to the specified <code>String</code>.
	 */
	public void setInitializer(String initializer) { this.initializer = initializer; }
	
	boolean isAddableTo(ChildBearingNode node)
	{
		return (node instanceof ClassNode);
	}
}