package ca.ubc.cs.elide.nodes;

import ca.ubc.cs.elide.*;
import ca.ubc.cs.elide.visitors.*;
import ca.ubc.cs.elide.utils.*;
import java.util.*;
import java.io.*;

/**
 * A <code>PackageNode</code> provides information on and access to a single Java package.
 **/
 
public class PackageNode extends ChildBearingNode
{	
	public PackageNode() {}
	public PackageNode(String name) { super(name); }
			
	/**
	 * Returns an array of <code>ClassNode</code> objects representing the classes contained by this package.
	 **/
	public ClassNode[] getClasses() { return (ClassNode[]) getChildren(ClassNode.class); }
	
	/**
	* Returns a <code>ClassNode</code> object representing a class contained by this package, given its name.
	**/
	public ClassNode getClass(String name) { return (ClassNode) getChild(ClassNode.class, name); }
	
	/**
	 * Throws a RuntimeException if called - included for compatibility.
	 **/
	public ClassNode getDeclaringClass() { throw new RuntimeException("Tried to get declaring class of package"); }
	
	/**
	 * Returns this <code>PackageNode</code> object (included for compatibility).
	 **/
	public PackageNode getPackage() { return this; }
	
	boolean isAddableTo(ChildBearingNode node)
	{
		return (node instanceof StartNode);
	}
}