package ca.ubc.cs.elide.nodes;

import java.util.*;
import ca.ubc.cs.elide.transformations.*;
import ca.ubc.cs.elide.visitors.*;
import ca.ubc.cs.elide.utils.*;
import ca.ubc.cs.elide.*;

/**
 * A <code>StartNode</code> represents the root of a parse tree - this tree is made up of representations of all of the
 * packages, classes, methods, and so on, being processed by ELIDE.  <code>StartNode</code>s are used internally
 * within ELIDE, 
 */
public class StartNode extends ChildBearingNode
{		
	/** maintains all active transforms; for internal use **/
	public transient TransformCollector transformCollector;
	
	public StartNode()
	{
		add(new PackageNode(""));
	}
	
	/** Returns an array of <code>PackageNode</code> objects representing the packages contained in this tree **/
	public PackageNode[] getPackages() { return (PackageNode[]) getChildren(PackageNode.class); }
	public PackageNode getPackage(String name) { return (PackageNode) getChild(PackageNode.class, name); }
	
	/**
	 * Returns a <code>PackageNode</code> representing the default (unnamed)
	 * package.
	 **/
	public PackageNode getDefaultPackage()
	{
		return getPackage("");
	}
	
	/**
	 * Returns a new tree which is the result of applying all the transformations
	 * in this <code>StartNode</code>; for internal use.
	 **/
	public StartNode transform()
	{
		StartNode clone = (StartNode)clone();
		
		//reset transform map
		transformCollector = new TransformCollector();
		transformCollector.visit(clone);
		clone.transformCollector = transformCollector;
		
		System.out.println("Resolving transformation dependencies...");
		Iterator dependencies = Transform.resolveDependencies().iterator();
		
		Map transformMap = transformCollector.getTransforms();
				
		System.out.println("Performing source-to-source transforms...");
		while(dependencies.hasNext())
		{
			Dependency dep = (Dependency) dependencies.next();
			List transformList = (List)transformMap.get(dep.getTransformName().toLowerCase());
			if(transformList != null)
			{
				System.out.println("Executing " + dep.getTransformName() + "#" + dep.getMethodName() + "...");
				for(int i = 0; i < transformList.size(); i++)
				{
					TransformNode t = (TransformNode) transformList.get(i);
					t.invoke(dep.getMethodName());
				}
			}
		}
		
		return clone;
	}
}