package ca.ubc.cs.elide.nodes;

import java.util.*;
import java.lang.reflect.*;
import ca.ubc.cs.elide.utils.*;
import ca.ubc.cs.elide.visitors.*;
import ca.ubc.cs.elide.utils.*;
import ca.ubc.cs.elide.*;

import ca.ubc.cs.elide.transformations.*;

/**
 * A <code>TransformNode</node> represents a single invocation
 * of a transformation within source code.
 *<p>
 * Note that <code>TransformNode</code>s should be added with
 * {@link ChildBearingNode#addTransform}, not with {@link ChildBearingNode#add}.
 **/
public class TransformNode extends ParseNode
{
	public String transformName;
	public List args;
		
	public TransformNode(String transformName)
	{
		this(transformName, new ArrayList());
	}
		
	public TransformNode(String transformName, List args)
	{
		this.transformName = transformName;
		this.args = args;
		
		Transform.registerTransform(transformName);
	}
	
	boolean isAddableTo(ChildBearingNode node)
	{
		return true;
	}

	public String getName()
	{
		return transformName;
	}
	
	public String toString()
	{
		return transformName + "<" + args + ">";
	}

	/**
	 * invokes the transform; for internal use
	 **/
	public void invoke(String methodName)
	{		
		ParseNode nodeTarget = getParent();
		
		Transform transformTarget = Transform.getInstance(transformName);
		
		Class[] classes = new Class[args.size() + 1];
		Object[] arguments = new Object[args.size() + 1];
		
		classes[0] = nodeTarget.getClass();
		arguments[0] = nodeTarget;
		
		int i = 1;
		Iterator iterator = args.iterator();
		while(iterator.hasNext())
		{
			classes[i] = String.class;
			arguments[i] = iterator.next();
			i++;
		}

		Method method = findMethod(classes, methodName, transformTarget);
		if(method == null)
		{
			method = findMethod(new Class[]{nodeTarget.getClass(), List.class}, methodName, transformTarget);
			arguments = new Object[]{nodeTarget, args};
		}
		
		try
		{
			method.invoke(transformTarget, arguments);
		}
		catch(InvocationTargetException e)
		{
			e.getTargetException().printStackTrace();
		}
		catch(Exception e)
		{
			System.out.println("Couldn't find matching method for " + transformTarget.getClass().getName() +
				"#" + methodName + " with args " + args);
		}	
	}
	
	private Method findMethod(Class[] classes, String methodName, Transform transformTarget)
	{
		while(classes[0] != Object.class)
		{
			try
			{
				return transformTarget.getClass().getMethod(methodName, classes);
			}
			catch(Exception e)
			{}
				
			classes[0] = classes[0].getSuperclass();
		}
		
		return null;
	}
}