package ca.ubc.cs.elide.sable;

import ca.ubc.cs.elide.*;
import ca.ubc.cs.elide.nodes.*;
import ca.ubc.cs.elide.utils.*;

import j11.*;
import java.io.*;
import java.util.*;
import ca.mcgill.sable.gagnon.java.node.*;
import ca.mcgill.sable.gagnon.java.lexer.*;
import ca.mcgill.sable.gagnon.java.parser.*;
import ca.mcgill.sable.gagnon.java.analysis.*;

public class PrettyPrinter extends DepthFirstAdapter
{		
	public PrettyPrinter() {}
	
	/***  blocks ***/
	
	public void caseAClassBody(AClassBody node)
	{
		inAClassBody(node);
		lineList(node.getClassBodyDeclaration());
		outAClassBody(node);
	}

    public void caseAConstructorBody(AConstructorBody node)
    {
		inAConstructorBody(node);
		printLine();
		traverse(node.getExplicitConstructorInvocation());
		lineList(node.getBlockStatement());
		outAConstructorBody(node);
    }
	
    public void caseAInterfaceBody(AInterfaceBody node)
    {
		inAInterfaceBody(node);
		lineList(node.getInterfaceMemberDeclaration());
		outAInterfaceBody(node);
    }
	
    public void caseABlock(ABlock node)
    {
        inABlock(node);
		lineList(node.getBlockStatement());
		outABlock(node);
    }

    public void caseASwitchBlock(ASwitchBlock node)
    {
        inASwitchBlock(node);
		lineList(node.getSwitchBlockStatementGroup());
		lineList(node.getSwitchLabel());
		outASwitchBlock(node);
    }

	public void caseADdslLiteralLiteral(ADdslLiteralLiteral node)
	{
		String text = node.toString().trim();
		print(InternalUtils.quote(text));
	}	

	/*** by default, all blocks get indented ***/
	
	static Class[] indent = {
		AClassBody.class,
		AConstructorBody.class,
		AInterfaceBody.class,
		ABlock.class,
		ASwitchBlock.class,
	};

	public void defaultIn(Node node)
	{
		if(isMember(node, indent))
			startBlock();
	}
	
	public void defaultOut(Node node)
	{
		if(isMember(node, indent))
			endBlock();
	}
	
	/*** semicolons, parens, etc, eat whitespace to their left and sometimes right***/

	static Class[] leftTrim = {
		TSemicolon.class,
		TLParenthese.class,
		TRParenthese.class,
		TLBracket.class,
		TRBracket.class,
		TDot.class,
		TComma.class,
	};
	
	static Class[] rightTrim = {
		TLParenthese.class,
		TLBracket.class,
		TDot.class,
	};
	
	public void defaultCase(Node node)
    {
		if(isMember(node, leftTrim))
			trim();
		
		print(node);
		
		if(isMember(node, rightTrim))
			trim();
    }
	
	/***  helper routines ***/
	
	private StringBuffer buffer = new StringBuffer();
	private int indentLevel = 0;
		
	public String getBuffer()
	{
		return buffer.toString();
	}
	
	public String resetBuffer()
	{
		String saved = getBuffer();
		buffer = new StringBuffer();
		return saved;
	}
	
	protected void print(Object printable)
	{
		buffer.append(printable.toString());
	}
	
	protected void printSpace()
	{
		buffer.append(" ");
	}
	
	protected void printLine()
	{
		buffer.append("\n");
		for(int i = 0; i < indentLevel; i++)
			buffer.append("\t");
	}
	
	protected void trim()
	{
		buffer = new StringBuffer(buffer.toString().trim());
	}
	
	protected void traverse(Node node)
	{
		if(node != null)
			node.apply(this);
	}
	
	protected void wordList(List list)
	{
		Iterator i = list.iterator();
		while(i.hasNext())
		{
			traverse((Node)i.next());
			printSpace();
		}
	}
	
	protected void lineList(List list)
	{
		Iterator i = list.iterator();
		while(i.hasNext())
		{
			printLine();
			traverse((Node)i.next());
		}
	}
	
	/** indenting style defined here **/
	
	protected void startBlock()
	{
		printLine();
		print("{");
		tabIn();
	}
	
	protected void endBlock()
	{
		tabOut();
		printLine();
		print("}");
		printLine();
	}	
	
	/** indentation **/
	
	protected void tabIn() { indentLevel++; }
	protected void tabOut() { indentLevel--; }
	
	protected int getIndent() { return indentLevel; }
	
	protected void setIndent(int indent)
	{
		indentLevel = indent;
	}
	
	protected int resetIndent()
	{
		int saved = indentLevel;
		indentLevel = 0;
		return saved;
	}
	
	private boolean isMember(Node node, Class[] classes)
	{
		for(int i = 0; i < classes.length; i++)
			if(node.getClass() == classes[i])
				return true;
			
		return false;
	}	
}
