package ca.ubc.cs.elide.utils;

import ca.ubc.cs.elide.*;
import ca.ubc.cs.elide.transformations.*;
import ca.ubc.cs.elide.nodes.*;
import ca.ubc.cs.elide.sable.*;
import ca.ubc.cs.elide.transforms.*;
import ca.ubc.cs.elide.visitors.*;
import ca.ubc.cs.elide.crosscutting.*;

import java.util.*;
import java.io.*;

public class Main
{
	private List sourceFiles;
	private List aspectFiles;
	
	private StartNode baseTree;
	private StartNode aspectTree;
	
	private Merger baseMerger;
	private Merger aspectMerger;

	private static String outputDir = "output";
		
	private static PackageTransformFactory transformFactory = new PackageTransformFactory();
	
	public Main(List sourceFiles, List aspectFiles)
	{
		this.sourceFiles = sourceFiles;
		this.aspectFiles = aspectFiles;

		baseMerger = new Merger(new StartNode());
		aspectMerger = new Merger(new StartNode());
		
		baseTree = load(sourceFiles, baseMerger);
		aspectTree = load(aspectFiles, aspectMerger);
	}
	
	public StartNode load(List sources, Merger merger)
	{
		StartNode tree = new StartNode();
		Iterator i = sources.iterator();
		while(i.hasNext())
			((SourceFile)i.next()).load(merger);
		return merger.into;
	}
	
	public StartNode transform()
	{
		return (Crosscutter.crosscut(baseTree,aspectTree)).transform();
	}
		
	public static void main(String[] args)
	{
		if(args.length == 0)
			System.out.println("usage: java ca.ubc.cs.elide.utils.Main [-d outputDir] [-P modifierPackage] files...");
		else
		{
			List sources = new ArrayList();
			List aspects = new ArrayList();
			List transforms = new ArrayList();
			
			for(int i = 0; i < args.length; i++)
			{
				if(args[i].equals("-d"))
					outputDir = args[++i];
				else if(args[i].equals("-P"))
					transformFactory.addPackage(args[++i]);
				else if(InternalUtils.endsWithAspectSuffix(args[i]))
					aspects.add(new SourceFile(args[i]));
				else
					sources.add(new SourceFile(args[i]));
			}
			
			FilePrinter printer = new FilePrinter(outputDir);
			
			Transform.setFactory(transformFactory);
			long start = (new Date()).getTime();
			Main main = new Main(sources, aspects);
//			System.out.println((new Date()).getTime()-start);
			
			start = (new Date()).getTime();
			StartNode tree = main.transform();
//			System.out.println((new Date()).getTime()-start);
			
			start = (new Date()).getTime();
			printer.visit(tree);
//			System.out.println((new Date()).getTime()-start);
		}
	}
}